angular.module('beamng.apps')
    .directive('distancetimer', ['$log', 'bngApi', 'StreamsManager', 'UiUnits', function ($log, bngApi, StreamsManager, UiUnits) {
        return {
            template:
            
            '<div style="margin: 10px; background-color: rgba(255, 255, 255, 0.4); padding:10px; box-shadow: 0px 0px 5px 0px rgba(0,0,0,0.75); border-bottom: solid 2px ' + "{{status_color}}" + '; border-bottom: solid 2px ' + "{{status_color}}" + '">'
                + '<div style="width:100%; height:100%; background-color: rgba(255, 255, 255, 0.9); padding:10px; " layout="column">'

                    + '<div layout="row" layout-align="space-between">'
       
                        + '<div layout="column">'                     
                            + '<div style="font-size: 1.5em; text-align:center;">' + "{{ timer | date:'mm:ss:sss'  }}" + '</div>' 
                            + '<div style="font-size: 0.7em; text-align:center;">' + "{{ status_color == 'red' ? 'TIME' : 'LAST TIME' }}" + '</div>' 
                        + '</div>'

                        + '<div layout="column">' 
                            + '<div style="font-size: 1.5em; text-align:center;">' + "{{ best_time | date:'mm:ss:sss'  }}" + '</div>' 
                            + '<div style="font-size: 0.7em; text-align:center;">' + "BEST TIME" + '</div>' 
                        + '</div>'

                    + '</div>'

                    + '<div layout="column">' 
                        + '<div style=" display:' + "{{settings ? 'visible' : 'none'}}" + ';" >'
                            + '<md-slider flex min=0 max="'+ "{{ units.label == 'meters' ? 3495 : 1.995 }}" +'" ng-model="goalStart" step="' + "{{ units.label == 'meters' ? 5 : 0.005 }}" + '"></md-slider>'
                            + '<md-slider style="margin-top:-10px;" flex min=0 max="'+ "{{ units.label == 'meters' ? 3500 : 2 }}" + '" ng-model="goalEnd" step="' + "{{ units.label == 'meters' ? 5 : 0.005 }}" + '"></md-slider></div>' 
                            + '<md-button class="md-flat" style=" text-transform: none; width: 40%; margin: -10px auto -10px;  font-size: 0.7em;" ng-click="buttonClick()">' +"{{ goalStart }} - {{ goalEnd }} {{ units.label }}" + '</md-button>' 
                        + '</div>'

                        + '<div layout="row" layout-align="space-between" style="margin-top:15px;">' 
                            + '<div flex="100" style="background-color: #b2b3b5; hegiht: 5px; overflow: hidden;">'
                                + '<div style="font-size: 6px; background-color: #555; width:' + "{{ (( (distance-goalStart)/(goalEnd-goalStart) )*100) <0 ? '0' : (( (distance-goalStart)/(goalEnd-goalStart) )*100)  }}" + '%; ">&nbsp;'
                                + '</div>'
                        + '</div>'             
                    + '</div>'         

                + '</div>'
            + '</div>',


    replace: true,
    restrict: 'EA',

    controller: ['$scope', function ($scope) {
        
    
        
        $scope.$watch('goalStart', function (newVal, oldVal) {
            // clear values if goalStart is changed
            $scope.best_time = 0;
            $scope.start_time = 0;
            $scope.timer = 0;
            $scope.ready = false;
            $scope.status_color = 'transparent';

            // don't allow to have goalEnd smaller than goalStart 
            if ($scope.goalEnd < $scope.goalStart)
            {
                $scope.goalEnd = $scope.goalStart;
            }

        }, true);        
       
        $scope.$watch('goalEnd', function (newVal, oldVal) {
            // clear values if goalEnd is changed
            $scope.best_time = 0;
            $scope.start_time = 0;
            $scope.timer = 0;
            $scope.ready = false;
            $scope.status_color = 'transparent';
            
            // don't allow to have goalEnd smaller than goalStart 
            if ($scope.goalEnd < $scope.goalStart)
            {
                $scope.goalStart = $scope.goalEnd;
            }

        }, true);

        $scope.settings = false;
        $scope.buttonClick = function () {
            if ($scope.settings) {
                $scope.settings = false;
            } else {
                $scope.settings = true;
            }
        };
}],


    link: function (scope, element, attrs) {
        'use strict';
        StreamsManager.add(['electrics']);

        scope.$on('$destroy', function () {
            $log.debug('<distancetimer> destroyed');
            StreamsManager.remove(['electrics']);
        });

        var unitSystems = {
            imperial: {
                speedFactor: 2.236,
                label: 'miles'
            },
            metric: {
                speedFactor: 3.6,
                label: 'meters'
            }
        };
      
        
        // set values when app is added to screen for the first time
        if (UiUnits.speed(0).unit == 'mph')
        {
            scope.units = unitSystems.imperial; 
            scope.goalEnd = '0.25';
        }
        else
        {
            scope.units = unitSystems.metric; 
            scope.goalEnd = '400';            
        }
        
        scope.speed = NaN;     
        scope.goalStart = 0;
        scope.status_color = 'transparent';
        scope.timer = 0;
        scope.best_time = 0;
        scope.ready = false;    
        scope.distance = 0;  


        scope.$on('SettingsChanged', function (event, data) {
            var uiUnits = data.values.uiUnits;

            if (unitSystems[uiUnits]) {
                scope.$evalAsync(function () {
                    scope.units = unitSystems[uiUnits];
                });
                if (uiUnits == 'imperial'){
                    // default value in miles
                    scope.goalEnd = '0.250';
                }
                else if (uiUnits == 'metric'){
                    // default value in meters
                    scope.goalEnd = '400';
                }
                scope.goalStart = 0;
                
            } else {
                $log.warn('<distancetimer> Attempted to change to unknown units %s', uiUnits);
            }
        });

        scope.$on('streamsUpdate', function (event, streams) {
            scope.$evalAsync(function () {               
         
                // get speed
                var speedMs = streams.electrics.airspeed;
                scope.speed = ('0000' + Math.round(speedMs * scope.units.speedFactor)).slice(-4);                
                
                if (scope.ready && scope.speed !=0) // check if we are ready and moving
                {                    
                    // get current position
                    scope.current_pos = streams.sensors.position;
                    
                    // calculate travelled distance
                    var calcDistance = Math.sqrt(Math.pow((scope.current_pos.x - scope.last_pos.x), 2) + Math.pow((scope.current_pos.y - scope.last_pos.y), 2) + Math.pow((scope.current_pos.z - scope.last_pos.z), 2));
                    
                    //convert meters to miles if necessary
                    scope.distance += scope.units.label == 'meters' ?  calcDistance : 0.000621371192 * calcDistance;
                    
                    // save current position into last position for next distance mesuring
                    scope.last_pos = scope.current_pos;
                    
                    if (scope.distance < scope.goalStart) // if we haven't reached starting distance, save current time
                    {
                        scope.start_time = performance.now();
                        scope.status_color = 'orange';
                    }
                    else if (scope.distance >= scope.goalStart && scope.distance < scope.goalEnd) // if we are between start and end distance, measure time!
                    {
                        scope.status_color = 'red';
                        var newTime = performance.now();
                        scope.timer += newTime - scope.start_time;
                        scope.start_time = newTime;
                    }        
                    else // if we reached end distance, stop timer, save results
                    {
                        scope.ready = false;
                        scope.distance = 0;
                        scope.status_color = 'transparent';

                        if (scope.best_time > scope.timer || scope.best_time == 0) {
                            scope.best_time = scope.timer;
                        }     
                    } 
                }                
                else if (scope.speed == 0) { // ready to start, show green bar, prepare position and save current time
                    scope.status_color = 'green';
                    scope.ready = true;
                    scope.last_pos = streams.sensors.position;
                    scope.distance = 0;
                    scope.timer = 0;
                    scope.start_time = performance.now();
                }
                else {
                    scope.status_color = 'transparent';
                    scope.ready = false;                   
                }

            });
        });
    }
}
}])